@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003,2005,2006
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003,2005,2006,2008
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -107,7 +107,7 @@ extern void _gst_pop_temporaries_dictionary (OOP dictionaryOOP)
   ATTRIBUTE_HIDDEN;
 
 extern OOP _gst_and_symbol ATTRIBUTE_HIDDEN;
-extern OOP _gst_as_scaled_decimal_scale_symbol ATTRIBUTE_HIDDEN;
+extern OOP _gst_as_scaled_decimal_radix_scale_symbol ATTRIBUTE_HIDDEN;
 extern OOP _gst_bad_return_error_symbol ATTRIBUTE_HIDDEN;
 extern OOP _gst_boolean_symbol ATTRIBUTE_HIDDEN;
 extern OOP _gst_byte_array_out_symbol ATTRIBUTE_HIDDEN;
@@ -135,6 +135,8 @@ extern OOP _gst_primitive_symbol ATTRIBUTE_HIDDEN;
 extern OOP _gst_repeat_symbol ATTRIBUTE_HIDDEN;
 extern OOP _gst_self_smalltalk_symbol ATTRIBUTE_HIDDEN;
 extern OOP _gst_self_symbol ATTRIBUTE_HIDDEN;
+extern OOP _gst_short_symbol ATTRIBUTE_HIDDEN;
+extern OOP _gst_ushort_symbol ATTRIBUTE_HIDDEN;
 extern OOP _gst_smalltalk_symbol ATTRIBUTE_HIDDEN;
 extern OOP _gst_smalltalk_namespace_symbol ATTRIBUTE_HIDDEN;
 extern OOP _gst_start_execution_symbol ATTRIBUTE_HIDDEN;
@@ -149,6 +151,7 @@ extern OOP _gst_times_repeat_symbol ATTRIBUTE_HIDDEN;
 extern OOP _gst_to_by_do_symbol ATTRIBUTE_HIDDEN;
 extern OOP _gst_to_do_symbol ATTRIBUTE_HIDDEN;
 extern OOP _gst_true_symbol ATTRIBUTE_HIDDEN;
+extern OOP _gst_uchar_symbol ATTRIBUTE_HIDDEN;
 extern OOP _gst_uint_symbol ATTRIBUTE_HIDDEN;
 extern OOP _gst_ulong_symbol ATTRIBUTE_HIDDEN;
 extern OOP _gst_undeclared_symbol ATTRIBUTE_HIDDEN;
@@ -188,13 +191,11 @@ extern const char *_gst_get_scope_kind (scope_type scope)
    SE is untouched and FALSE is returned.  */
 extern mst_Boolean _gst_find_variable (symbol_entry * se,
 				       tree_node list)
-  ATTRIBUTE_PURE
   ATTRIBUTE_HIDDEN;
 
 /* This converts a C string to a symbol and stores it in the symbol
    table.  */
 extern OOP _gst_intern_string (const char *str)
-  ATTRIBUTE_PURE
   ATTRIBUTE_HIDDEN;
 
 /* This makes an Array with an element for each instance variable
@@ -216,18 +217,21 @@ extern OOP _gst_make_class_variable_dictionary (const char * variableNames,
 extern OOP _gst_make_pool_array (const char * poolNames)
   ATTRIBUTE_HIDDEN;
 
-/* This resolves to an Association the variable binding constant expressed
-   by the LIST parse tree node.  Unless DECLARE_TEMPORARY is false,
-   temporary variables may be automatically declared.  */
-extern OOP _gst_find_variable_binding (tree_node list,
-				       mst_Boolean declare_temporary)
+/* This resolves the variable binding constant expressed by the LIST parse
+   tree node.  Unless DECLARE_TEMPORARY is false, temporary variables
+   may be automatically declared.  */
+extern tree_node _gst_find_variable_binding (tree_node list)
+  ATTRIBUTE_HIDDEN;
+
+/* This returns the dictionary in which to define an undeclared variable
+   binding.  */
+extern OOP _gst_get_undeclared_dictionary ()
   ATTRIBUTE_PURE
   ATTRIBUTE_HIDDEN;
 
 /* This converts the Smalltalk String STRINGOOP into a Symbol and
    return the converted Symbol.  */
 extern OOP _gst_intern_string_oop (OOP stringOOP)
-  ATTRIBUTE_PURE
   ATTRIBUTE_HIDDEN;
 
 /* Given a symbol, this routine returns a C string that corresponds to the
@@ -296,7 +300,7 @@ extern int _gst_declare_name (const char *name,
 			      mst_Boolean allowDup)
   ATTRIBUTE_HIDDEN;
 
-/* Computes the number of selectors that a message named SYMBOLOOP
+/* Computes the number of arguments that a message named SYMBOLOOP
    expects.  */
 extern int _gst_selector_num_args (OOP symbolOOP)
   ATTRIBUTE_PURE
